/*
 * Copyright (c) 2015 N. Billard.
 */
package com.massa.mrules.demos.spring.ann;

import java.io.InputStream;

import org.springframework.context.annotation.Bean;

import com.massa.mrules.builder.XmlRuleEngineConfigHolder;
import com.massa.mrules.exception.MConfigurationException;
import com.massa.mrules.extensions.spring.AbstractMRulesFactoryBean.AdditionalProperty;
import com.massa.mrules.extensions.spring.MRulesFactoryBean;
import com.massa.mrules.factory.xml.RichXMLFactory;
import com.massa.mrules.set.IMruleExecutionSet;
import com.massa.util.classloader.ResourceLoader;

/**
 * Spring configuration class for Spring Annotation Demo, showing both solutions to manage your beans.
 */
public class SpringAnnConfig {

	/**
	 * Creates a rule engine, managed by Spring via the MRules Factory Bean.
	 */
	@Bean(name = "factRuleEngine")
	public MRulesFactoryBean getFactRuleEngine() throws MConfigurationException {
		final MRulesFactoryBean fact = new MRulesFactoryBean();
		fact.setUri("TEST");
		fact.setFactoryImpl(RichXMLFactory.class.getName());
		fact.setConfigHolderImpl(XmlRuleEngineConfigHolder.class.getName());
		fact.setXmlFile("rules_by_fact.xml");

		// Additional properties. Not useful here, just for the example.
		fact.addProperty(new AdditionalProperty("custom_property_1_key", "custom_property_1_value"));
		fact.addProperty(new AdditionalProperty("custom_property_2_key", "custom_property_2_value"));

		return fact;
	}

	/**
	 * Creates a rule engine, managed by Spring as a Singleton. So, no need to use the builder here.
	 */
	@Bean(name = "annRuleEngine")
	public IMruleExecutionSet getAnnRuleEngine() throws MConfigurationException {
		final InputStream xml = ResourceLoader.getResourceAsStream("rules_by_ann.xml");
		final RichXMLFactory factory = new RichXMLFactory();
		return factory.read(xml);
	}
}
